@@ -1,5 +1,6 @@
 from gateways.apis.api_about import ApiAbout
 from gateways.apis.api_account import ApiAccount
+from gateways.apis.api_account_cost import ApiAccountCost
 from gateways.apis.api_achievements import ApiAchievements
 from gateways.apis.api_agent_connector import ApiAgentConnector
 from gateways.apis.api_artifacts_repository import ApiArtifactsRepository
@@ -22,6 +23,7 @@ class ApiRoot:
     def __init__(self, api_address: str, space: str = None, version: str = None):
         self.about = ApiAbout(api_address, version)
         self.account = ApiAccount(api_address, version)
+        self.account_cost = ApiAccountCost(api_address, version)
         self.achievements = ApiAchievements(api_address, version)
         self.artifacts_repository = ApiArtifactsRepository(api_address, version)
         self.blueprint = ApiBlueprint(api_address, version)
