@@ -19,7 +19,7 @@ from gateways.common.cs18_api_requests import (
     UpdateSpaceRequest,
     ProductionEnvironment, DebuggingServiceValue, SpaceCloudAccountInfraSettingsRequest,
     AddK8SComputeServiceToSpaceRequest, UpdateK8SComputeServiceInSpaceRequest, AwsCloudProviderSettingsModel,
-    NewUserSurveyRequest)
+    NewUserSurveyRequest, AddSpaceTerraformModuleRequest, UpdateSpaceTerraformModuleRequest)
 from gateways.common.cs18_api_responses import (AccountStatusResponse, BlueprintFileResponse, BlueprintResponse,
                                                 CatalogForGetAllResponse, CatalogForGetResponse, CloudAccountResponse,
                                                 CreateProductionResponse, CreateSandboxResponse, GetSpaceResponse,
@@ -33,7 +33,8 @@ from gateways.common.cs18_api_responses import (AccountStatusResponse, Blueprint
                                                 DirectAwsSshResponse, DirectAzureSshResponse, DirectRdpResponse,
                                                 ParameterStoreItem, RepositoryProviderResponse,
                                                 RepositoryRelationshipResponse, AccountExtraDetails, CustomTagResponse,
-                                                TerraformModuleDescriptorResponse)
+                                                TerraformModuleDescriptorResponse, CustomTagOverrideResponse,
+                                                SpaceTerraformModuleResponse)
 from gateways.n_session import NSession
 from gateways.the_gateway import TheGateway
 from gateways.utils import GatewayUtils, Utils
@@ -585,6 +586,11 @@ class Colony:
         response = self.session.put(url=url)
         GatewayUtils.handle_response(response=response, return_codes=[200, 202])
 
+    def delete_green(self, production_id: str):
+        url = f'{self._api_root.production.delete_green(production_id=production_id)}'
+        response = self.session.delete(url=url)
+        GatewayUtils.handle_response(response=response, return_codes=[202])
+
     def promote_green(self, production_id: str):
         url = self._api_root.production.promote_green(production_id=production_id)
         response = self.session.put(url=url)
@@ -647,7 +653,7 @@ class Colony:
         method_url = self._api_root.sandbox.sandbox_by_id(sandbox_id=sandbox_id)
         resp = self.session.delete(url=method_url)
 
-        if resp.status_code in [408, 429, 504]:
+        if resp.status_code in [408, 423, 429, 504]:
             if retry:
                 time.sleep(30)  # need to make sure that the next call will not use the cached timedout error
                 self.end_sandbox(sandbox_id, False)
@@ -771,11 +777,14 @@ class Colony:
             self,
             repository_details: BlueprintRepositoryDetails,
             deployment_type: str = None,
+            space_name: str = None,
             wait_for_cache_refresh: bool = False,
     ):
         print(
             f'repository_url: {repository_details.repository_url} access_token: {repository_details.access_token}')
-        method_url = self._api_root.spaces.space_repositories(space_name=self.space)
+        if space_name is None:
+            space_name = self.space
+        method_url = self._api_root.spaces.space_repositories(space_name=space_name)
         request = {
             "repository_url": repository_details.repository_url,
             "access_token": repository_details.access_token,
@@ -823,8 +832,11 @@ class Colony:
                 subject="blueprints ready",
             )
 
-    def remove_repository_from_space(self, wait_for_cache_refresh=False):
-        method_url = self._api_root.spaces.space_repositories(space_name=self.space)
+    def remove_repository_from_space(self, wait_for_cache_refresh=False, space_name: str = None,
+                                     deployment_type: str = None):
+        if space_name is None:
+            space_name = self.space
+        method_url = self._api_root.spaces.space_repositories(space_name=space_name, deployment_type=deployment_type)
         resp = self.session.delete(url=method_url)
         GatewayUtils.handle_response(response=resp, return_codes=[200])
 
@@ -1525,6 +1537,43 @@ class Colony:
         resp = self.session.delete(url=method_url)
         GatewayUtils.handle_response(response=resp, return_codes=[200])
 
+    def get_terraform_modules_available_for_space(self):
+        method_url = self._api_root.spaces.available_space_terraform_modules(space_name=self.space)
+
+        response = self.session.get(url=method_url)
+        GatewayUtils.handle_response(response=response, return_codes=[200])
+        json_result = json.loads(response.text)
+        return [Converters.create_terraform_module_descriptor_response(terraform_module)
+                for terraform_module in json_result]
+
+    def get_space_terraform_modules(self) -> List[SpaceTerraformModuleResponse]:
+        method_url = self._api_root.spaces.attached_space_terraform_modules(space_name=self.space)
+
+        response = self.session.get(url=method_url)
+        GatewayUtils.handle_response(response=response, return_codes=[200])
+        json_result = json.loads(response.text)
+        return [Converters.create_space_terraform_module_response(item)
+                for item in json_result]
+
+    def add_space_terraform_module(self, request: AddSpaceTerraformModuleRequest):
+        method_url = self._api_root.spaces.attached_space_terraform_modules(space_name=self.space)
+
+        response = self.session.post(url=method_url, json=Utils.get_as_json(request))
+        GatewayUtils.handle_response(response=response, return_codes=[200])
+
+    def update_space_terraform_module(self, module_alias: str, request: UpdateSpaceTerraformModuleRequest):
+        method_url = self._api_root.spaces.attached_space_terraform_module(space_name=self.space,
+                                                                           module_alias=module_alias)
+
+        response = self.session.put(url=method_url, json=Utils.get_as_json(request))
+        GatewayUtils.handle_response(response=response, return_codes=[200])
+
+    def delete_space_terraform_module(self, module_alias: str):
+        method_url = self._api_root.spaces.attached_space_terraform_module(space_name=self.space,
+                                                                           module_alias=module_alias)
+        response = self.session.delete(url=method_url)
+        GatewayUtils.handle_response(response=response, return_codes=[200])
+
     def get_terraform_versions(self) -> List[str]:
         method_url = self._api_root.settings.terraform_versions()
         res = self.session.get(url=method_url)
@@ -1568,6 +1617,67 @@ class Colony:
         res = self.session.delete(url=method_url)
         GatewayUtils.handle_response(response=res, return_codes=[200])
 
+    def get_space_custom_tags(self, space_name: str) -> List[CustomTagOverrideResponse]:
+        method_url = self._api_root.spaces.custom_space_tags(space_name)
+        res = self.session.get(url=method_url)
+        GatewayUtils.handle_response(response=res, return_codes=[200])
+        custom_tags_json = json.loads(res.text)
+        return [Converters.create_custom_tag_override_response(custom_tag) for custom_tag in custom_tags_json]
+
+    def create_space_custom_tag(self, space_name: str, name: str, value: str):
+        method_url = self._api_root.spaces.custom_space_tags(space_name)
+        request = {
+            "name": name,
+            "value": value
+        }
+        res = self.session.post(url=method_url, json=request)
+        GatewayUtils.handle_response(response=res, return_codes=[200])
+
+    def update_space_custom_tag(self, space_name: str, name: str, value: str):
+        method_url = self._api_root.spaces.custom_space_tag(space_name, name)
+        request = {
+            "value": value
+        }
+        res = self.session.put(url=method_url, json=request)
+        GatewayUtils.handle_response(response=res, return_codes=[200])
+
+    def delete_space_custom_tag(self, space_name: str, name: str):
+        method_url = self._api_root.spaces.custom_space_tag(space_name, name)
+        res = self.session.delete(url=method_url)
+        GatewayUtils.handle_response(response=res, return_codes=[200])
+
+    def get_blueprint_custom_tags(self, space_name: str,
+                                  blueprint_name: str, deployment_type: str) -> List[CustomTagOverrideResponse]:
+        method_url = self._api_root.spaces.custom_blueprint_tags(space_name, blueprint_name, deployment_type)
+        res = self.session.get(url=method_url)
+        GatewayUtils.handle_response(response=res, return_codes=[200])
+        custom_tags_json = json.loads(res.text)
+        return [Converters.create_custom_tag_override_response(custom_tag) for custom_tag in custom_tags_json]
+
+    def create_blueprint_custom_tag(self, space_name: str,
+                                    blueprint_name: str, deployment_type: str, name: str, value: str):
+        method_url = self._api_root.spaces.custom_blueprint_tags(space_name, blueprint_name, deployment_type)
+        request = {
+            "name": name,
+            "value": value
+        }
+        res = self.session.post(url=method_url, json=request)
+        GatewayUtils.handle_response(response=res, return_codes=[200])
+
+    def update_blueprint_custom_tag(self, space_name: str,
+                                    blueprint_name: str, deployment_type: str, name: str, value: str):
+        method_url = self._api_root.spaces.custom_blueprint_tag(space_name, blueprint_name, name, deployment_type)
+        request = {
+            "value": value
+        }
+        res = self.session.put(url=method_url, json=request)
+        GatewayUtils.handle_response(response=res, return_codes=[200])
+
+    def delete_blueprint_custom_tag(self, space_name: str, blueprint_name: str, deployment_type: str, name: str):
+        method_url = self._api_root.spaces.custom_blueprint_tag(space_name, blueprint_name, name, deployment_type)
+        res = self.session.delete(url=method_url)
+        GatewayUtils.handle_response(response=res, return_codes=[200])
+
 
 class SandboxesHandler:
     def __init__(self):
