@@ -5,6 +5,8 @@ import abc
 from typing import List
 import dateutil.parser
 
+from common.assertion_helper import AssertionHelper
+
 
 class AccessLink:
     def __init__(self, protocol: str, link: str):
@@ -165,8 +167,8 @@ class TerraformModuleDescriptor:
         description = self.description == other.description
         terraform_version = self.terraform_version == other.terraform_version
         enable_auto_tagging = self.enable_auto_tagging == other.enable_auto_tagging
-        exclude_from_tagging = self.compare_arrays(self.exclude_from_tagging, other.exclude_from_tagging)
-        allowed_spaces = self.compare_arrays(self.allowed_spaces, other.allowed_spaces)
+        exclude_from_tagging = AssertionHelper.compare_arrays(self.exclude_from_tagging, other.exclude_from_tagging)
+        allowed_spaces = AssertionHelper.compare_arrays(self.allowed_spaces, other.allowed_spaces)
 
         current_cs = \
             sorted(self.compute_services, key=lambda x: x.cloud_account_name + x.compute_service_name, reverse=True)
@@ -181,14 +183,15 @@ class TerraformModuleDescriptor:
                enable_auto_tagging and exclude_from_tagging and allowed_spaces and compute_services and \
                outputs and inputs
 
-    @staticmethod
-    def compare_arrays(first, second):
 
-        if first is None or second is None:
-            return first == second
+class SpaceTerraformModuleInput:
+    def __init__(self, name: str, value: str, overridable: bool) -> None:
+        self.name = name
+        self.value = value
+        self.overridable = overridable
 
-        first_sorted = \
-            sorted(first)
-        second_sorted = \
-            sorted(second)
-        return first_sorted == second_sorted
+    def clone(self):
+        """
+        :return: SpaceTerraformModuleInput
+        """
+        return SpaceTerraformModuleInput(name=self.name, value=self.value, overridable=self.overridable)
