@@ -2,7 +2,7 @@ from typing import List, Optional
 from datetime import datetime
 import dateutil.parser
 from gateways.common.cs18_api_classes import AccessLink, TerraformModuleDescriptor, TerraformModuleComputeService, \
-    TerraformModuleInput, TerraformModuleOutput
+    TerraformModuleInput, TerraformModuleOutput, SpaceTerraformModuleInput
 from gateways.common.cs18_api_errors import SandboxError, Error
 from gateways.common.cs18_api_requests import AwsCloudProviderSettingsModel
 from gateways.common.cs18_api_responses import (GetSpacesResponse, CreateAccountFinalizeResponse, ParameterStoreItem,
@@ -43,7 +43,9 @@ from gateways.common.cs18_api_responses import (GetSpacesResponse, CreateAccount
                                                 ArtifactRepoResponse, CatalogForGetAllResponse, CatalogForGetResponse,
                                                 SpaceCloudAccount, SpaceAdmin, SandboxOutput, ServiceTfProgressResponse,
                                                 TerraformModuleDescriptorResponse, CustomTagResponse,
-                                                ServiceTfLogsResponse, ServiceTfLogInfo)
+                                                CustomTagOverrideResponse, ServiceTfLogsResponse,
+                                                ServiceTfLogInfo, SpaceTerraformModuleResponse, BlueprintTagResponse,
+                                                AccountRepositoryItemResponse)
 
 
 class Converters:
@@ -99,6 +101,12 @@ class Converters:
             else [],
             artifacts=catalog.get("artifacts", {}),
             errors=catalog.get("errors"),
+            tags=[
+                Converters._create_blueprint_tag_response(tag)
+                for tag in catalog["tags"]
+            ]
+            if catalog["tags"]
+            else []
         )
 
     @staticmethod
@@ -120,6 +128,12 @@ class Converters:
             else [],
             artifacts=catalog.get("artifacts", {}),
             errors=catalog.get("errors"),
+            tags=[
+                Converters._create_blueprint_tag_response(tag)
+                for tag in catalog["tags"]
+            ]
+            if catalog["tags"]
+            else []
         )
 
     @staticmethod
@@ -201,6 +215,15 @@ class Converters:
                 for app in blueprint["services"]
             ]
             if blueprint["services"]
+            else [],
+            environment_type=blueprint["environment_type"]
+            if "environment_type" in blueprint
+            else "",
+            tags=[
+                Converters._create_blueprint_tag_response(tag)
+                for tag in blueprint["tags"]
+            ]
+            if blueprint["tags"]
             else []
         )
 
@@ -674,6 +697,14 @@ class Converters:
 
         return BlueprintServiceResponse(name=service["name"], service_type=service["type"])
 
+    @staticmethod
+    def _create_blueprint_tag_response(tag: dict) -> BlueprintTagResponse:
+
+        return BlueprintTagResponse(name=tag["name"],
+                                    default_value=tag["default_value"],
+                                    possible_values=tag["possible_values"],
+                                    description=tag["description"])
+
     @staticmethod
     def _create_blueprint_cloud_region_response(
             region: dict
@@ -835,7 +866,7 @@ class Converters:
             details=repo["details"],
             status=repo["status"],
             account_id=repo["account_id"],
-            repositories=[Converters.create_account_repository_response for r in repo["repositories"]],
+            repositories=[Converters.create_account_repository_item_response(r) for r in repo["repositories"]],
             errors=[Error(e) for e in repo["errors"]],
         )
 
@@ -849,7 +880,6 @@ class Converters:
         return AccountRepositoryResponse(
             repository_id=repo["id"],
             name=repo["name"],
-            provider_id=repo["provider_id"],
             provider_name=repo["provider_name"],
             allow_sharing=repo["allow_sharing"],
             open_access=repo["open_access"],
@@ -858,6 +888,29 @@ class Converters:
             repository_url=repo["repository_url"],
             connection_details=repo["connection_details"],
             errors=[Error(e) for e in repo["errors"]],
+            repository_relationships=
+            [Converters.create_repository_relationship_response(r) for r in
+             repo["repository_relationships"]]
+        )
+
+    @staticmethod
+    def create_account_repository_item_response(repo: dict) -> AccountRepositoryItemResponse:
+        """
+        Returns AccountRepositoryItemResponse based on provided dictionary
+        :param repo: dict
+        :return: AccountRepositoryResponse
+        """
+        return AccountRepositoryItemResponse(
+            repository_id=repo["id"],
+            name=repo["name"],
+            allow_sharing=repo["allow_sharing"],
+            open_access=repo["open_access"],
+            is_valid=repo["is_valid"],
+            repository_url=repo["repository_url"],
+            errors=[Error(e) for e in repo["errors"]],
+            repository_relationships=
+            [Converters.create_repository_relationship_response(r) for r in
+             repo["repository_relationships"]]
         )
 
     @staticmethod
@@ -873,7 +926,6 @@ class Converters:
             space_id=repo["space_id"],
             deployment_type=repo["deployment_type"],
             branch=repo["branch"],
-            account_id=repo["account_id"],
             is_valid=repo["is_valid"],
             errors=[Error(e) for e in repo["errors"]],
         )
@@ -945,31 +997,41 @@ class Converters:
 
         tf_tagging_dict = logs_dict["preparetags"]
         if tf_tagging_dict:
-            tf_tagging = ServiceTfLogInfo(location=tf_tagging_dict["location"])
+            tf_tagging = ServiceTfLogInfo(location=tf_tagging_dict["location"],
+                                          start_time=tf_tagging_dict["start_time"],
+                                          duration=tf_tagging_dict["duration"])
         else:
             tf_tagging = None
 
         tf_init_dict = logs_dict["tfinit"]
         if tf_init_dict:
-            tf_init = ServiceTfLogInfo(location=tf_init_dict["location"])
+            tf_init = ServiceTfLogInfo(location=tf_init_dict["location"],
+                                       start_time=tf_init_dict["start_time"],
+                                       duration=tf_init_dict["duration"])
         else:
             tf_init = None
 
         tf_plan_dict = logs_dict["tfplan"]
         if tf_plan_dict:
-            tf_plan = ServiceTfLogInfo(location=tf_plan_dict["location"])
+            tf_plan = ServiceTfLogInfo(location=tf_plan_dict["location"],
+                                       start_time=tf_plan_dict["start_time"],
+                                       duration=tf_plan_dict["duration"])
         else:
             tf_plan = None
 
         tf_apply_dict = logs_dict["tfapply"]
         if tf_apply_dict:
-            tf_apply = ServiceTfLogInfo(location=tf_apply_dict["location"])
+            tf_apply = ServiceTfLogInfo(location=tf_apply_dict["location"],
+                                        start_time=tf_apply_dict["start_time"],
+                                        duration=tf_apply_dict["duration"])
         else:
             tf_apply = None
 
         tf_destroy_dict = logs_dict["tfdestroy"]
         if tf_destroy_dict:
-            tf_destroy = ServiceTfLogInfo(location=tf_destroy_dict["location"])
+            tf_destroy = ServiceTfLogInfo(location=tf_destroy_dict["location"],
+                                          start_time=tf_destroy_dict["start_time"],
+                                          duration=tf_destroy_dict["duration"])
         else:
             tf_destroy = None
 
@@ -1158,6 +1220,17 @@ class Converters:
             created_date=data['created_date']
         )
 
+    @staticmethod
+    def create_custom_tag_override_response(data: dict) -> CustomTagOverrideResponse:
+        return CustomTagOverrideResponse(
+            name=data['name'],
+            value=data['value'],
+            modified_by=data['modified_by'],
+            last_modified=data['last_modified'],
+            created_by=data['created_by'],
+            created_date=data['created_date']
+        )
+
     @staticmethod
     def _create_sandbox_outputs(data: Optional[List]) -> Optional[List[SandboxOutput]]:
         if data is None:
@@ -1230,3 +1303,24 @@ class Converters:
             "allowed_spaces": module.allowed_spaces
         }
         return request
+
+    @staticmethod
+    def create_space_terraform_module_response(data: dict) -> SpaceTerraformModuleResponse:
+        return SpaceTerraformModuleResponse(
+            module_alias=data["module_alias"],
+            module_name=data["module_name"],
+            description=data["description"],
+            cloud_account_name=data["cloud_account_name"],
+            compute_service_name=data["compute_service_name"],
+            region=data["region"],
+            enable_logs=data["enable_logs"],
+            inputs=[Converters.convert_space_terraform_module_input(input_item) for input_item in data["inputs"]]
+        )
+
+    @staticmethod
+    def convert_space_terraform_module_input(input_item: dict) -> SpaceTerraformModuleInput:
+        return SpaceTerraformModuleInput(
+            name=input_item["name"],
+            value=input_item["value"],
+            overridable=input_item["overridable"]
+        )
