@@ -9,7 +9,7 @@ from enum import Enum
 from typing import List, Dict, Optional
 
 from gateways.common.cs18_api_classes import AccessLink, TerraformModuleDescriptor, \
-    TerraformModuleInput, TerraformModuleComputeService, TerraformModuleOutput
+    TerraformModuleInput, TerraformModuleComputeService, TerraformModuleOutput, SpaceTerraformModuleInput
 from gateways.common.cs18_api_errors import SandboxError, Error
 
 
@@ -19,6 +19,14 @@ class BlueprintSourceResponse:
         self.branch = branch
 
 
+class BlueprintTagResponse:
+    def __init__(self, name: str, default_value: str, possible_values: [], description: str):
+        self.name = name
+        self.default_value = default_value
+        self.possible_values = possible_values
+        self.description = description
+
+
 class AccountStatusUserDetails:
     def __init__(self, user_details: {}):
         self.last_name = user_details["last_name"]
@@ -114,6 +122,7 @@ class CatalogForGetResponse:
             links: List[HyperlinkResponse],
             artifacts: Dict[str, str],
             errors: [],
+            tags: List[BlueprintTagResponse],
     ):
         self.errors = errors
         self.links = links
@@ -125,6 +134,7 @@ class CatalogForGetResponse:
         self.clouds = clouds
         self.applications = applications
         self.artifacts = artifacts
+        self.tags = tags
 
 
 class CatalogForGetAllResponse:
@@ -139,6 +149,7 @@ class CatalogForGetAllResponse:
             links: List[HyperlinkResponse],
             artifacts: Dict[str, str],
             errors: [],
+            tags: List[BlueprintTagResponse],
     ):
         self.links = links
         self.url = url
@@ -149,6 +160,7 @@ class CatalogForGetAllResponse:
         self.clouds = clouds
         self.artifacts = artifacts
         self.errors = errors
+        self.tags = tags
 
 
 class BlueprintApplicationResponse:
@@ -237,7 +249,9 @@ class BlueprintResponse:
             inputs: List[BlueprintInputResponse],
             errors: List[Error],
             is_sample,
-            services: List[BlueprintServiceResponse]
+            services: List[BlueprintServiceResponse],
+            environment_type: str,
+            tags: List[BlueprintTagResponse]
     ):
         self.errors = errors
         self.is_sample = is_sample
@@ -253,6 +267,8 @@ class BlueprintResponse:
         self.artifacts = artifacts
         self.inputs = inputs
         self.services = services
+        self.environment_type = environment_type
+        self.tags = tags
 
 
 class SourceControlDetailsResponse:
@@ -489,13 +505,17 @@ class ServiceTfProgressResponse:
 
 class ServiceTfLogInfo:
     def __init__(self,
-                 location: str):
+                 location: str,
+                 start_time: str,
+                 duration: str):
         self.location = location
+        self.start_time = start_time
+        self.duration = duration
 
 
 class ServiceTfLogsResponse:
     def __init__(self,
-                 tf_tagging: ServiceTfLogInfo= None,
+                 tf_tagging: ServiceTfLogInfo = None,
                  tf_init: ServiceTfLogInfo = None,
                  tf_plan: ServiceTfLogInfo = None,
                  tf_apply: ServiceTfLogInfo = None,
@@ -983,9 +1003,40 @@ class RepositoryResponse:
         self.branch = branch
 
 
+class AccountRepositoryResponse:
+    def __init__(self, repository_id: str, name: str, provider_name: str, allow_sharing: str,
+                 open_access: str, account_id: str, is_valid: bool, connection_details, repository_url: str,
+                 errors: List[Error], repository_relationships):
+        self.repository_url = repository_url
+        self.repository_id = repository_id
+        self.name = name
+        self.provider_name = provider_name
+        self.allow_sharing = allow_sharing
+        self.open_access = open_access
+        self.account_id = account_id
+        self.is_valid = is_valid
+        self.errors = errors
+        self.connection_details = connection_details
+        self.repository_relationships = repository_relationships
+
+
+class AccountRepositoryItemResponse:
+    def __init__(self, repository_id: str, name: str, allow_sharing: str,
+                 open_access: str, is_valid: bool, repository_url: str,
+                 errors: List[Error], repository_relationships):
+        self.repository_url = repository_url
+        self.repository_id = repository_id
+        self.name = name
+        self.allow_sharing = allow_sharing
+        self.open_access = open_access
+        self.is_valid = is_valid
+        self.errors = errors
+        self.repository_relationships = repository_relationships
+
+
 class RepositoryProviderResponse:
     def __init__(self, provider_id: str, name: str, details, status: str, account_id: str,
-                 repositories, errors: List[Error]):
+                 repositories: List[AccountRepositoryItemResponse], errors: List[Error]):
         self.provider_id = provider_id
         self.errors = errors
         self.account_id = account_id
@@ -996,28 +1047,10 @@ class RepositoryProviderResponse:
         self.repositories = repositories
 
 
-class AccountRepositoryResponse:
-    def __init__(self, repository_id: str, name: str, provider_id: str, provider_name: str, allow_sharing: str,
-                 open_access: str, account_id: str, is_valid: bool, connection_details, repository_url: str,
-                 errors: List[Error]):
-        self.repository_url = repository_url
-        self.repository_id = repository_id
-        self.name = name
-        self.provider_id = provider_id
-        self.provider_name = provider_name
-        self.allow_sharing = allow_sharing
-        self.open_access = open_access
-        self.account_id = account_id
-        self.is_valid = is_valid
-        self.errors = errors
-        self.connection_details = connection_details
-
-
 class RepositoryRelationshipResponse:
     def __init__(self, repo_relationship_id: str, space_id: str, deployment_type: str,
-                 account_id: str, branch: str, is_valid: bool, errors: List[Error]):
+                 branch: str, is_valid: bool, errors: List[Error]):
         self.repo_relationship_id = repo_relationship_id
-        self.account_id = account_id
         self.space_id = space_id
         self.deployment_type = deployment_type
         self.branch = branch
@@ -1255,7 +1288,24 @@ class CustomTagResponse:
         self.created_date = created_date
         self.created_by = created_by
 
-        
+
+class CustomTagOverrideResponse:
+    def __init__(self,
+                 name: str,
+                 value: str,
+                 modified_by: str,
+                 last_modified: datetime,
+                 created_by: str,
+                 created_date: datetime,
+                 ):
+        self.name = name
+        self.value = value
+        self.modified_by = modified_by
+        self.last_modified = last_modified
+        self.created_date = created_date
+        self.created_by = created_by
+
+
 class TerraformModuleDescriptorResponse(TerraformModuleDescriptor):
     def __init__(self, module_name: str = None, module_repo_name: str = None, module_root_path: str = None,
                  description: str = None, terraform_version: str = None, enable_auto_tagging: bool = None,
@@ -1278,3 +1328,25 @@ class TerraformModuleDescriptorResponse(TerraformModuleDescriptor):
         if not isinstance(other, TerraformModuleDescriptor):
             return NotImplemented
         return super(TerraformModuleDescriptorResponse, self).__eq__(other)
+
+
+class SpaceTerraformModuleResponse:
+    def __init__(
+            self,
+            module_alias: str,
+            module_name: str,
+            description: Optional[str],
+            cloud_account_name: str,
+            compute_service_name: str,
+            region: Optional[str],
+            enable_logs: bool,
+            inputs: Optional[List[SpaceTerraformModuleInput]]
+    ):
+        self.module_alias = module_alias
+        self.module_name = module_name
+        self.description = description
+        self.cloud_account_name = cloud_account_name
+        self.compute_service_name = compute_service_name
+        self.region = region
+        self.enable_logs = enable_logs
+        self.inputs = inputs
