@@ -1,10 +1,11 @@
 import datetime
 import time
-from typing import Tuple, Optional
+from typing import Tuple, Optional, Callable
 
 from gateways.colony_client import Colony
 from gateways.common.cs18_api_errors import SandboxNotFound, SandboxEndingFailed
-from gateways.common.cs18_api_responses import SandboxResponse, ProductionBlueResponse, ProductionGreenResponse
+from gateways.common.cs18_api_responses import SandboxResponse, ProductionBlueResponse, ProductionGreenResponse, \
+    BlueprintResponse
 from gateways.common.cs18_api_statuses import SandboxStatus, SandboxDeploymentStatus, ProductionUpdateStatus
 from gateways.utils import SandboxUtils, Utils
 
@@ -128,6 +129,15 @@ class CommonWaiters:
                               max_seconds=max_seconds,
                               subject=f'Was production green exposure updated to {exposure_value}%?')
 
+    def wait_until_production_green_deleted(self,
+                                      production_id: str,
+                                      interval_sec: int = 5,
+                                      max_retries: int = 60):
+        Utils.wait_for(func=lambda: self._production_green_deleted(production_id=production_id),
+                       interval_sec=interval_sec,
+                       max_retries=max_retries,
+                       subject='Production green {} deleted'.format(production_id))
+
     def wait_until_production_deleted(self,
                                       production_id: str,
                                       interval_sec: int = 5,
@@ -330,6 +340,11 @@ class CommonWaiters:
         success, blue_env, _ = self._safely_get_production_details(production_id=production_id)
         return success and not blue_env
 
+    def _production_green_deleted(self, production_id: str) -> bool:
+        success, blue_env, _ = self._safely_get_production_details(production_id=production_id)
+        # After green deletion is completed update status should be 'None'
+        return success and blue_env.update_status == 'None'
+
     def _safely_get_production_details(self, production_id: str, handle_timeout: bool = True) \
             -> Tuple[bool, Optional[ProductionBlueResponse], str]:
         try:
@@ -340,6 +355,16 @@ class CommonWaiters:
                 raise
             return False, None, "Timeout Error"
 
+    def _safely_get_production_green_details(self, production_id: str, handle_timeout: bool = True) \
+            -> Tuple[bool, Optional[ProductionGreenResponse], str]:
+        try:
+            production = self._api.get_production_green_details(production_id=production_id)
+            return True, production, ""
+        except TimeoutError:
+            if not handle_timeout:
+                raise
+            return False, None, "Timeout Error"
+
     def _production_green_deployed(self, production_id: str) -> Tuple[bool, str]:
         success, production, failure_reason = self._safely_get_production_green_details(production_id=production_id)
         if not success:
@@ -382,12 +407,29 @@ class CommonWaiters:
 
         return True, f"Yes, green exposure is {exposure}%"
 
-    def _safely_get_production_green_details(self, production_id: str, handle_timeout: bool = True) \
-            -> Tuple[bool, Optional[ProductionGreenResponse], str]:
-        try:
-            production = self._api.get_production_green_details(production_id=production_id)
-            return True, production, ""
-        except TimeoutError:
-            if not handle_timeout:
-                raise
-            return False, None, "Timeout Error"
+    def wait_until_sandbox_output_resolved(self, sandbox_id: str, output_name: str,
+                                           interval_sec: int = 10, max_seconds = 60):
+        Utils.wait_until_time(
+            func=lambda: self._sandbox_output_resolved(sandbox_id=sandbox_id, output_name=output_name),
+            interval_sec=interval_sec,
+            max_seconds=max_seconds,
+            subject="waiting for output {} of sandbox {}".format(output_name, sandbox_id),
+        )
+
+    def _sandbox_output_resolved(self, sandbox_id: str, output_name: str):
+        sandbox_details = self._api.get_sandbox_details(sandbox_id=sandbox_id)
+        return any([o for o in sandbox_details.outputs if o.name == output_name and o.value is not None])
+
+    def wait_until_blueprint_exist(self, blueprint_name: str, check_extra: Callable[[BlueprintResponse], bool] = None,
+                                   interval_sec: int = 1, max_seconds: int = 15):
+        Utils.wait_until_time(
+            func=lambda: self._blueprint_exist(blueprint_name, check_extra),
+            interval_sec=interval_sec,
+            max_seconds=max_seconds,
+            subject="waiting for blueprint '{}' to be in blueprint list".format(blueprint_name),
+        )
+
+    def _blueprint_exist(self, blueprint_name: str, func: Callable[[BlueprintResponse], bool] = None):
+        return any([b for b in self._api.list_blueprints()
+                    if b.name == blueprint_name and
+                    (True if not func else func(b))])
